package alfred.web.http

import alfred.web.core.BuildId
import alfred.web.core.Builds
import alfred.web.core.GitRunner
import alfred.web.core.Handles
import alfred.web.core.ScriptRunner
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("build")
class Endpoints(
    val builds: Builds,
    val security: Security,
    val scriptRunner: ScriptRunner,
    val gitRunner: GitRunner,
    val handles: Handles
) {

    @GetMapping(
        "{build}/info",
        produces = [MediaType.APPLICATION_JSON_VALUE]
    )
    fun info(
        @PathVariable("build")
        build: BuildId,
        @RequestParam("key")
        key: String?
    ) = security.requireKey(build, key) {
        it
    }

    @PostMapping(
        "{build}/trigger",
        produces = [MediaType.APPLICATION_JSON_VALUE]
    )
    fun trigger(
        @PathVariable("build")
        build: BuildId,
        @RequestParam("key")
        key: String?,
        @RequestParam("rev")
        rev: String
    ) = security.requireKey(build, key) {
        val info = if (builds.buildConfig(build).script != null) {
            scriptRunner.run(build, rev)
        } else {
            gitRunner.run(build, rev)
        }

        ResponseEntity.ok(
            mapOf(
                "log" to info.logFile.backingFile.absolutePath,
                "pid" to info.pid
            )
        )
    }

    @GetMapping(
        "{build}/handles",
        produces = [MediaType.APPLICATION_JSON_VALUE]
    )
    fun handles(
        @PathVariable("build")
        build: BuildId,
        @RequestParam("key")
        key: String?
    ) = security.requireKey(build, key) {
        handles.active(build)
    }

}

